import typing
import asyncio
import serial
from forge.acquisition import CONFIGURATION, LayeredConfiguration
from forge.acquisition.control.base import BaseControl
from forge.acquisition.cutsize import CutSize


class Control(BaseControl):
    CONTROL_TYPE = "impactor_output"

    class _OutputCutSize(CutSize):
        class Active(CutSize.Active):
            async def automatic_activation(self, now: float = None) -> bool:
                if not await super().automatic_activation(now):
                    return False

                if self.size == CutSize.Size.PM1:
                    self.control.serial_port.write(b"h\n")
                elif self.size == CutSize.Size.PM10:
                    self.control.serial_port.write(b"l\n")

                return True

    def __init__(self):
        super().__init__()
        self.cycle = self._OutputCutSize(LayeredConfiguration(CONFIGURATION.get("ACQUISITION.CUT_SIZE")))
        for a in self.cycle.active:
            a.control = self
        self.serial_port = serial.Serial("/dev/serial/by-id/usb-FTDI_TTL232R_FTFOHUR6-if00-port0", 9600)

    async def run(self):
        await self.cycle.automatic_activation()


def main():
    from forge.acquisition.control.run import launch

    launch(Control)

